# -*- coding: utf-8 -*-
import unittest
from gtts.tokenizer.pre_processors import (
    tone_marks,
    end_of_line,
    abbreviations,
    word_sub,
)


class TestPreProcessors(unittest.TestCase):
    def test_tone_marks(self):
        _in = "lorem!ipsum?"
        _out = "lorem! ipsum? "
        self.assertEqual(tone_marks(_in), _out)

    def test_end_of_line(self):
        _in = """test-
ing"""
        _out = "testing"
        self.assertEqual(end_of_line(_in), _out)

    def test_abbreviations(self):
        _in = "jr. sr. dr."
        _out = "jr sr dr"
        self.assertEqual(abbreviations(_in), _out)

    def test_word_sub(self):
        _in = "Esq. Bacon"
        _out = "Esquire Bacon"
        self.assertEqual(word_sub(_in), _out)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import pytest
from gtts.lang import tts_langs, _extra_langs, _fallback_deprecated_lang
from gtts.langs import _main_langs

"""Test language list"""


def test_main_langs():
    """Fetch languages successfully"""
    # Safe to assume 'en' (English) will always be there
    scraped_langs = _main_langs()
    assert "en" in scraped_langs


def test_deprecated_lang():
    """Test language deprecation fallback"""
    with pytest.deprecated_call():
        assert _fallback_deprecated_lang("en-gb") == "en"


if __name__ == "__main__":
    pytest.main(["-x", __file__])



# -*- coding: utf-8 -*-
import pytest
import re
import os
from click.testing import CliRunner
from gtts.cli import tts_cli

# Need to look into gTTS' log output to test proper instantiation
# - Use testfixtures.LogCapture() b/c TestCase.assertLogs() needs py3.4+
# - Clear 'gtts' logger handlers (set in gtts.cli) to reduce test noise
import logging
from testfixtures import LogCapture

logger = logging.getLogger("gtts")
logger.handlers = []


"""Test options and arguments"""


def runner(args, input=None):
    return CliRunner().invoke(tts_cli, args, input)


def runner_debug(args, input=None):
    return CliRunner().invoke(tts_cli, args + ["--debug"], input)


# <text> tests
def test_text_no_text_or_file():
    """One of <test> (arg) and <file> <opt> should be set"""
    result = runner_debug([])

    assert "<file> required" in result.output
    assert result.exit_code != 0


def test_text_text_and_file(tmp_path):
    """<test> (arg) and <file> <opt> should not be set together"""
    filename = tmp_path / "test_and_file.txt"
    filename.touch()

    result = runner_debug(["--file", str(filename), "test"])

    assert "<file> can't be used together" in result.output
    assert result.exit_code != 0


def test_text_empty(tmp_path):
    """Exit on no text to speak (via <file>)"""
    filename = tmp_path / "text_empty.txt"
    filename.touch()

    result = runner_debug(["--file", str(filename)])

    assert "No text to speak" in result.output
    assert result.exit_code != 0


# <file> tests
def test_file_not_exists():
    """<file> should exist"""
    result = runner_debug(["--file", "notexist.txt", "test"])

    assert "No such file or directory" in result.output
    assert result.exit_code != 0


# <all> tests
@pytest.mark.net
def test_all():
    """Option <all> should return a list of languages"""
    result = runner(["--all"])

    # One or more of "  xy: name" (\n optional to match the last)
    # Ex. "<start>  xx: xxxxx\n  xx-yy: xxxxx\n  xx: xxxxx<end>"

    assert re.match(r"^(?:\s{2}(\w{2}|\w{2}-\w{2}): .+\n?)+$", result.output)
    assert result.exit_code == 0


# <lang> tests
@pytest.mark.net
def test_lang_not_valid():
    """Invalid <lang> should display an error"""
    result = runner(["--lang", "xx", "test"])

    assert "xx' not in list of supported languages" in result.output
    assert result.exit_code != 0


@pytest.mark.net
def test_lang_nocheck():
    """Invalid <lang> (with <nocheck>) should display an error message from gtts"""
    with LogCapture() as lc:
        result = runner_debug(["--lang", "xx", "--nocheck", "test"])

        log = str(lc)

    assert "lang: xx" in log
    assert "lang_check: False" in log
    assert "Unsupported language 'xx'" in result.output
    assert result.exit_code != 0


# Param set tests
@pytest.mark.net
def test_params_set():
    """Options should set gTTS instance arguments (read from debug log)"""
    with LogCapture() as lc:
        result = runner_debug(
            ["--lang", "fr", "--tld", "es", "--slow", "--nocheck", "test"]
        )

        log = str(lc)

    assert "lang: fr" in log
    assert "tld: es" in log
    assert "lang_check: False" in log
    assert "slow: True" in log
    assert "text: test" in log
    assert result.exit_code == 0


# Test all input methods
pwd = os.path.dirname(__file__)

# Text for stdin ('-' for <text> or <file>)
textstdin = """stdin
test
123"""

# Text for stdin ('-' for <text> or <file>) (Unicode)
textstdin_unicode = u"""你吃饭了吗？
你最喜欢哪部电影？
我饿了，我要去做饭了。"""

# Text for <text> and <file>
text = """Can you make pink a little more pinkish can you make pink a little more pinkish, nor can you make the font bigger?
How much will it cost the website doesn't have the theme i was going for."""

textfile_ascii = os.path.join(pwd, "input_files", "test_cli_test_ascii.txt")

# Text for <text> and <file> (Unicode)
text_unicode = u"""这是一个三岁的小孩
在讲述她从一系列照片里看到的东西。
对这个世界， 她也许还有很多要学的东西，
但在一个重要的任务上， 她已经是专家了：
去理解她所看到的东西。"""

textfile_utf8 = os.path.join(pwd, "input_files", "test_cli_test_utf8.txt")

"""
Method that mimics's LogCapture's __str__ method to make
the string in the comprehension a unicode literal for P2.7
https://github.com/Simplistix/testfixtures/blob/32c87902cb111b7ede5a6abca9b597db551c88ef/testfixtures/logcapture.py#L149
"""


def logcapture_str(lc):
    if not lc.records:
        return "No logging captured"

    return "\n".join([u"%s %s\n  %s" % r for r in lc.actual()])


@pytest.mark.net
def test_stdin_text():
    with LogCapture() as lc:
        result = runner_debug(["-"], textstdin)
        log = logcapture_str(lc)

    assert "text: %s" % textstdin in log
    assert result.exit_code == 0


@pytest.mark.net
def test_stdin_text_unicode():
    with LogCapture() as lc:
        result = runner_debug(["-"], textstdin_unicode)
        log = logcapture_str(lc)

    assert "text: %s" % textstdin_unicode in log
    assert result.exit_code == 0


@pytest.mark.net
def test_stdin_file():
    with LogCapture() as lc:
        result = runner_debug(["--file", "-"], textstdin)
        log = logcapture_str(lc)

    assert "text: %s" % textstdin in log
    assert result.exit_code == 0


@pytest.mark.net
def test_stdin_file_unicode():
    with LogCapture() as lc:
        result = runner_debug(["--file", "-"], textstdin_unicode)
        log = logcapture_str(lc)

    assert "text: %s" % textstdin_unicode in log
    assert result.exit_code == 0


@pytest.mark.net
def test_text():
    with LogCapture() as lc:
        result = runner_debug([text])
        log = logcapture_str(lc)

    assert "text: %s" % text in log
    assert result.exit_code == 0


@pytest.mark.net
def test_text_unicode():
    with LogCapture() as lc:
        result = runner_debug([text_unicode])
        log = logcapture_str(lc)

    assert "text: %s" % text_unicode in log
    assert result.exit_code == 0


@pytest.mark.net
def test_file_ascii():
    with LogCapture() as lc:
        result = runner_debug(["--file", textfile_ascii])
        log = logcapture_str(lc)

    assert "text: %s" % text in log
    assert result.exit_code == 0


@pytest.mark.net
def test_file_utf8():
    with LogCapture() as lc:
        result = runner_debug(["--file", textfile_utf8])
        log = logcapture_str(lc)

    assert "text: %s" % text_unicode in log
    assert result.exit_code == 0


@pytest.mark.net
def test_stdout():
    result = runner(["test"])

    # The MP3 encoding (LAME 3.99.5) used to leave a signature in the raw output
    # This no longer appears to be the case
    assert result.exit_code == 0


@pytest.mark.net
def test_file(tmp_path):
    filename = tmp_path / "out.mp3"

    result = runner(["test", "--output", str(filename)])

    # Check if files created is > 2k
    assert filename.stat().st_size > 2000
    assert result.exit_code == 0


if __name__ == "__main__":
    pytest.main(["-x", __file__])



# -*- coding: utf-8 -*-
from gtts.langs import _main_langs
from warnings import warn
import logging

__all__ = ["tts_langs"]

# Logger
log = logging.getLogger(__name__)
log.addHandler(logging.NullHandler())


def tts_langs():
    """Languages Google Text-to-Speech supports.

    Returns:
        dict: A dictionary of the type `{ '<lang>': '<name>'}`

            Where `<lang>` is an IETF language tag such as `en` or `zh-TW`,
            and `<name>` is the full English name of the language, such as
            `English` or `Chinese (Mandarin/Taiwan)`.

    The dictionary returned combines languages from two origins:

    - Languages fetched from Google Translate (pre-generated in :mod:`gtts.langs`)
    - Languages that are undocumented variations that were observed to work and
      present different dialects or accents.

    """
    langs = dict()
    langs.update(_main_langs())
    langs.update(_extra_langs())
    log.debug("langs: {}".format(langs))
    return langs


def _extra_langs():
    """Define extra languages.

    Returns:
        dict: A dictionary of extra languages manually defined.

            Variations of the ones generated in `_main_langs`,
            observed to provide different dialects or accents or
            just simply accepted by the Google Translate Text-to-Speech API.

    """
    return {
        # Chinese
        "zh-TW": "Chinese (Mandarin/Taiwan)",
        "zh": "Chinese (Mandarin)",
    }


def _fallback_deprecated_lang(lang):
    """Languages Google Text-to-Speech used to support.

    Language tags that don't work anymore, but that can
    fallback to a more general language code to maintain
    compatibility.

    Args:
        lang (string): The language tag.

    Returns:
        string: The language tag, as-is if not deprecated,
            or a fallback if it exits.

    Example:
        ``en-GB`` returns ``en``.
        ``en-gb`` returns ``en``.

    """

    deprecated = {
        # '<fallback>': [<list of deprecated langs>]
        "en": [
            "en-us",
            "en-ca",
            "en-uk",
            "en-gb",
            "en-au",
            "en-gh",
            "en-in",
            "en-ie",
            "en-nz",
            "en-ng",
            "en-ph",
            "en-za",
            "en-tz",
        ],
        "fr": ["fr-ca", "fr-fr"],
        "pt": ["pt-br", "pt-pt"],
        "es": ["es-es", "es-us"],
        "zh-CN": ["zh-cn"],
        "zh-TW": ["zh-tw"],
    }

    for fallback_lang, deprecated_langs in deprecated.items():
        if lang.lower() in deprecated_langs:
            msg = (
                "'{}' has been deprecated, falling back to '{}'. "
                "This fallback will be removed in a future version."
            ).format(lang, fallback_lang)

            warn(msg, DeprecationWarning)
            log.warning(msg)

            return fallback_lang

    return lang



# -*- coding: utf-8 -*-
import os
import pytest
from unittest.mock import Mock

from gtts.tts import gTTS, gTTSError
from gtts.langs import _main_langs
from gtts.lang import _extra_langs

# Testing all languages takes some time.
# Set TEST_LANGS envvar to choose languages to test.
#  * 'main': Languages extracted from the Web
#  * 'extra': Language set in Languages.EXTRA_LANGS
#  * 'all': All of the above
#  * <csv>: Languages tags list to test
# Unset TEST_LANGS to test everything ('all')
# See: langs_dict()


"""Construct a dict of suites of languages to test.
{ '<suite name>' : <list or dict of language tags> }

ex.: { 'fetch' : {'en': 'English', 'fr': 'French'},
       'extra' : {'en': 'English', 'fr': 'French'} }
ex.: { 'environ' : ['en', 'fr'] }
"""
env = os.environ.get("TEST_LANGS")
if not env or env == "all":
    langs = _main_langs()
    langs.update(_extra_langs())
elif env == "main":
    langs = _main_langs()
elif env == "extra":
    langs = _extra_langs()
else:
    env_langs = {l: l for l in env.split(",") if l}
    langs = env_langs


@pytest.mark.net
@pytest.mark.parametrize("lang", langs.keys(), ids=list(langs.values()))
def test_TTS(tmp_path, lang):
    """Test all supported languages and file save"""

    text = "This is a test"
    """Create output .mp3 file successfully"""
    for slow in (False, True):
        filename = tmp_path / "test_{}_.mp3".format(lang)
        # Create gTTS and save
        tts = gTTS(text=text, lang=lang, slow=slow, lang_check=False)
        tts.save(filename)

        # Check if files created is > 1.5
        assert filename.stat().st_size > 1500


@pytest.mark.net
def test_unsupported_language_check():
    """Raise ValueError on unsupported language (with language check)"""
    lang = "xx"
    text = "Lorem ipsum"
    check = True
    with pytest.raises(ValueError):
        gTTS(text=text, lang=lang, lang_check=check)


def test_empty_string():
    """Raise AssertionError on empty string"""
    text = ""
    with pytest.raises(AssertionError):
        gTTS(text=text)


def test_no_text_parts(tmp_path):
    """Raises AssertionError on no content to send to API (no text_parts)"""
    text = "                                                                                                          ..,\n"
    with pytest.raises(AssertionError):
        filename = tmp_path / "no_content.txt"
        tts = gTTS(text=text)
        tts.save(filename)


# Test write_to_fp()/save() cases not covered elsewhere in this file


@pytest.mark.net
def test_bad_fp_type():
    """Raise TypeError if fp is not a file-like object (no .write())"""
    # Create gTTS and save
    tts = gTTS(text="test")
    with pytest.raises(TypeError):
        tts.write_to_fp(5)


@pytest.mark.net
def test_save(tmp_path):
    """Save .mp3 file successfully"""
    filename = tmp_path / "save.mp3"
    # Create gTTS and save
    tts = gTTS(text="test")
    tts.save(filename)

    # Check if file created is > 2k
    assert filename.stat().st_size > 2000


@pytest.mark.net
def test_get_bodies():
    """get request bodies list"""
    tts = gTTS(text="test", tld="com", lang="en")
    body = tts.get_bodies()[0]
    assert "test" in body
    # \"en\" url-encoded
    assert "%5C%22en%5C%22" in body


def test_msg():
    """Test gTTsError internal exception handling
    Set exception message successfully"""
    error1 = gTTSError("test")
    assert "test" == error1.msg

    error2 = gTTSError()
    assert error2.msg is None


def test_infer_msg():
    """Infer message successfully based on context"""

    # Without response:

    # Bad TLD
    ttsTLD = Mock(tld="invalid")
    errorTLD = gTTSError(tts=ttsTLD)
    assert (
        errorTLD.msg
        == "Failed to connect. Probable cause: Host 'https://translate.google.invalid/' is not reachable"
    )

    # With response:

    # 403
    tts403 = Mock()
    response403 = Mock(status_code=403, reason="aaa")
    error403 = gTTSError(tts=tts403, response=response403)
    assert (
        error403.msg
        == "403 (aaa) from TTS API. Probable cause: Bad token or upstream API changes"
    )

    # 200 (and not lang_check)
    tts200 = Mock(lang="xx", lang_check=False)
    response404 = Mock(status_code=200, reason="bbb")
    error200 = gTTSError(tts=tts200, response=response404)
    assert (
        error200.msg
        == "200 (bbb) from TTS API. Probable cause: No audio stream in response. Unsupported language 'xx'"
    )

    # >= 500
    tts500 = Mock()
    response500 = Mock(status_code=500, reason="ccc")
    error500 = gTTSError(tts=tts500, response=response500)
    assert (
        error500.msg
        == "500 (ccc) from TTS API. Probable cause: Upstream API error. Try again later."
    )

    # Unknown (ex. 100)
    tts100 = Mock()
    response100 = Mock(status_code=100, reason="ddd")
    error100 = gTTSError(tts=tts100, response=response100)
    assert error100.msg == "100 (ddd) from TTS API. Probable cause: Unknown"


@pytest.mark.net
def test_WebRequest(tmp_path):
    """Test Web Requests"""

    text = "Lorem ipsum"

    """Raise gTTSError on unsupported language (without language check)"""
    lang = "xx"
    check = False

    with pytest.raises(gTTSError):
        filename = tmp_path / "xx.txt"
        # Create gTTS
        tts = gTTS(text=text, lang=lang, lang_check=check)
        tts.save(filename)


@pytest.mark.net
def test_timeout(tmp_path):
    # Check default timeout
    tts = gTTS(text="test")
    assert tts.timeout is None

    # Check passed in timeout
    timeout = 1.2
    tts = gTTS(text="test", timeout=timeout)
    assert tts.timeout == timeout

    # Make sure an exception is raised when a timeout occurs
    tts = gTTS(text="test", timeout=0.000001)
    filename = tmp_path / "save.mp3"
    with pytest.raises(gTTSError):
        tts.save(filename)


if __name__ == "__main__":
    pytest.main(["-x", __file__])
